function varargout = vidsync_vidgui(varargin)
%VIDSYNC_VIDGUI will control a gui to select the video and data frames to
%begin  syncronizing from.  The script takes two inputs for video frames
%and data frames, and returns two numbers corresponding to the frame to
%start syncronizing from.  The user can visually scroll thru video and data
%frames to select which ones to sync.
% 04/30/05 - A. McClung - Stanford University (1st stable version)

% VIDSYNC_VIDGUI M-file for vidsync_vidgui.fig
%      VIDSYNC_VIDGUI, by itself, creates a new VIDSYNC_VIDGUI or raises the existing
%      singleton*.
%
%      H = VIDSYNC_VIDGUI returns the handle to a new VIDSYNC_VIDGUI or the handle to
%      the existing singleton*.
%
%      VIDSYNC_VIDGUI('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in VIDSYNC_VIDGUI.M with the given input arguments.
%
%      VIDSYNC_VIDGUI('Property','Value',...) creates a new VIDSYNC_VIDGUI or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before vidsync_vidgui_OpeningFunction gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to vidsync_vidgui_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES
% Copyright 2002-2003 The MathWorks, Inc.
% Last Modified by GUIDE v2.5 29-Apr-2005 15:51:01
% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @vidsync_vidgui_OpeningFcn, ...
                   'gui_OutputFcn',  @vidsync_vidgui_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before vidsync_vidgui is made visible.
function vidsync_vidgui_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to vidsync_vidgui (see VARARGIN)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Variables that may need to be changed with video & data properties
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%NOTE: Some of these initial variables are set in both VIDSYNC_Rxx.M and VIDSYNC_VIDGUI.M, so be sure to change both.

% Rev 2.0 gets this from function call
%datasamp=1000;                  % The data sampling rate is 1kHz (1000Hz)

datacols=[4 5 6];               % Columns containing data for plotting
vidPos=[.1 .4 .8 .8];           % Video position and sizing
arraysize=7;                    % Size of the array to be shared by video and data
                                % NOTE: The array size affects where plots fall relative to the controls
axtext='XYZ';                   % Labels for data axes
axType='Accel';                 % Type of data
axYUnits='';                 % Units for axis
axcolor='bgrcmk';               % Colors for data plots
%Get the input data
mov=varargin{1};
data=varargin{2};
datasamp=varargin{3};
time=[1:size(data,1)]/datasamp';        % Create the time string for data plotting

%Prep the GUI w/data from the 1st frame
%Draw the 1st picture
figure(handles.figure1);
handles.VideoPlot=subplot(arraysize,1,1);       % Adjust array size for differet data
handles.VideoAxis=image(mov(1).cdata);  % Plot a frame of video
axis image off;                         % Correctly size the image
%Plot the data (3 force signals)
for i=datacols(1):datacols(1)+size(datacols,2)-1
    %Plot each column of data 
    eval(['h' num2str(i) '=subplot(' num2str(arraysize) ',1,' num2str(i+1) ');'])
    eval(['plot(time,data(:,' num2str(datacols(1+i-datacols(1))) '),''' axcolor(1+i-datacols(1)) ''');'])
    %Format the data plot
    axis tight; hold on; ylabel([axType ' ' axYUnits]); curaxis=axis;
    eval(['title(''' axtext(i-3) '-axis ' axType ' data'');'])
    grid on;
    %Add the vertical time bar and store handle
    eval(['handles.Bar' num2str(i) '=plot([0 0],[-3e3 3e3],''k'',''LineWidth'',2);'])
    %Restore the axis
    axis(curaxis);
    %Get the current sizing and position of each axis
    eval(['h' num2str(i) 'Pos=get(h' num2str(i) ',''Position'');'])
end
%Resize the video plot - this works better after all subplots are created
set(handles.VideoPlot,'Position',vidPos);
%SLOPPY - Reposition the data plots
vertdatapos=[.4 .28 .16];               % Vertical positioning of the data plots
                                        % NOTE: Adjust for different # data cols
for i=datacols(1):datacols(1)+size(datacols,2)-1
    eval(['set(h' num2str(i) ',''Position'',[h' num2str(i) 'Pos(1) ' ...
        num2str(vertdatapos(1+i-datacols(1))) ' h' num2str(i) 'Pos(3:4)]);'])
end

%Enter some personal variables into the handles structure
handles.MyVars.DMaxFrame=length(data);  % Number of data frames
handles.MyVars.VMaxFrame=size(mov,2);   % Number of video frames
handles.mov=mov;                        % Video frames
handles.data=data;                      % Data frames
handles.vidPos=vidPos;                  % Video position and sizing
handles.arraysize=arraysize;            % Size of the array for video, control and data
handles.datasamp=datasamp;              % Sampling rate of the data

% Choose default command line output for vidsync_vidgui
handles.output = hObject;
% Update handles structure
guidata(hObject, handles);
% UIWAIT makes vidsync_vidgui wait for user response (see UIRESUME)
uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = vidsync_vidgui_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Place the desired outputs in the output variable
varargout{1} = str2double(get(handles.VFrameNo,'String'));  %Output 1st video frame
varargout{2} = str2double(get(handles.DFrameNo,'String'));  %Output 1st data frame
close;          %Close the figure


%***********************************************************%
%***********************************************************%
%***************       VIDEO SECTION       *****************%
%***********************************************************%
%***********************************************************%

% --- Executes on button press in VBackBut.
function VBackBut_Callback(hObject, eventdata, handles)
% hObject    handle to VBackBut (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%Create easy to use variables
VFrameNo=str2double(get(handles.VFrameNo,'String'));
vidPos=handles.vidPos;
arraysize=handles.arraysize;
%Check for valid frame and update video frame
if (VFrameNo>1)
    VFrameNo=VFrameNo-1;
    %SLOPPY - Update the video frame and reposition the video plots
    handles.VideoPlot=subplot(arraysize,1,1,'Position',vidPos);
    handles.VideoAxis=image(handles.mov(VFrameNo).cdata);
    axis image off;
    %Store the updated variables back in the handles structure
    set(handles.VFrameNo,'String',num2str(VFrameNo));
end
% Update handles structure
guidata(hObject, handles);

% --- Executes on button press in ForwardBut.
function VForwardBut_Callback(hObject, eventdata, handles)
% hObject    handle to ForwardBut (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%Create easy to use variables
VFrameNo=str2double(get(handles.VFrameNo,'String'));
vidPos=handles.vidPos;
arraysize=handles.arraysize;
%Check for valid frame and update video frame
if (VFrameNo<handles.MyVars.VMaxFrame)
    VFrameNo=VFrameNo+1;
    %SLOPPY - Update the video frame and reposition the video plots
    handles.VideoPlot=subplot(arraysize,1,1,'Position',vidPos);
    handles.VideoAxis=image(handles.mov(VFrameNo).cdata);
    axis image off;
    %Store the updated variables back in the handles structure
    set(handles.VFrameNo,'String',num2str(VFrameNo));
end
% Update handles structure
guidata(hObject, handles);

function VFrameNo_Callback(hObject, eventdata, handles)
% hObject    handle to VFrameNo (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of VFrameNo as text
%        str2double(get(hObject,'String')) returns contents of VFrameNo as a double

%Create easy to use variables
VMaxFrame=handles.MyVars.VMaxFrame;
VFrameNo=str2double(get(handles.VFrameNo,'String'));
vidPos=handles.vidPos;
arraysize=handles.arraysize;
%Check for valid frame and update video frame
if (VFrameNo<1)
    VFrameNo=1;
else if (VFrameNo>VMaxFrame)
        VFrameNo=VMaxFrame;
    end
end
%SLOPPY - Update the video frame and reposition the video plots
handles.VideoPlot=subplot(arraysize,1,1,'Position',vidPos);
handles.VideoAxis=image(handles.mov(VFrameNo).cdata);
axis image off;
%Store the updated variables back in the handles structure
set(handles.VFrameNo,'String',num2str(VFrameNo));

% Update handles structure
guidata(hObject, handles);

% --- Executes during object creation, after setting all properties.
function VFrameNo_CreateFcn(hObject, eventdata, handles)
% hObject    handle to VFrameNo (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function VJumpNum_Callback(hObject, eventdata, handles)
% hObject    handle to VJumpNum (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of VJumpNum as text
%        str2double(get(hObject,'String')) returns contents of VJumpNum as a double


% --- Executes during object creation, after setting all properties.
function VJumpNum_CreateFcn(hObject, eventdata, handles)
% hObject    handle to VJumpNum (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in VJumpBut.
function VJumpBut_Callback(hObject, eventdata, handles)
% hObject    handle to VJumpBut (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
%Create easy to use variables
VMaxFrame=handles.MyVars.VMaxFrame;
VFrameNo=str2double(get(handles.VFrameNo,'String'));
vidPos=handles.vidPos;
VJumpNum=str2double(get(handles.VJumpNum,'String'));
arraysize=handles.arraysize;
%Check for valid frame and update video frame
VFrameNo=VFrameNo+VJumpNum;
if (VFrameNo<1)
    VFrameNo=1;
end
if (VFrameNo>VMaxFrame)
    VFrameNo=VMaxFrame;
end
%SLOPPY - Update the video frame and reposition the video plots
handles.VideoPlot=subplot(arraysize,1,1,'Position',vidPos);
handles.VideoAxis=image(handles.mov(VFrameNo).cdata);
axis image off;
%Store the updated variables back in the handles structure
set(handles.VFrameNo,'String',num2str(VFrameNo));
set(handles.VJumpNum,'String','0');

% Update handles structure
guidata(hObject, handles);


%***********************************************************%
%***********************************************************%
%*************       DATA SECTION       ********************%
%***********************************************************%
%***********************************************************%

% --- Executes on button press in DBackBut.
function DBackBut_Callback(hObject, eventdata, handles)
% hObject    handle to DBackBut (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%Create easy to use variables
DFrameNo=str2double(get(handles.DFrameNo,'String'));
%Check for valid frame and update vertical data time bar
if (DFrameNo>101)
    DFrameNo=DFrameNo-100;
    for i=4:6
        %Reposition vertical time bar in all data plots
        eval(['set(handles.Bar' num2str(i) ',''XData'',[DFrameNo DFrameNo]/handles.datasamp);'])
    end
    %Store the updated variables back in the handles structure
    set(handles.DFrameNo,'String',num2str(DFrameNo));
end
% Update handles structure
guidata(hObject, handles);


% --- Executes on button press in DForwardBut.
function DForwardBut_Callback(hObject, eventdata, handles)
% hObject    handle to DForwardBut (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%Create easy to use variables
DMaxFrame=handles.MyVars.DMaxFrame;
DFrameNo=str2double(get(handles.DFrameNo,'String'));
%Check for valid frame and update vertical data time bar
if (DFrameNo<DMaxFrame-99)
    DFrameNo=DFrameNo+100;
    for i=4:6
        %Reposition vertical time bar in all data plots
        eval(['set(handles.Bar' num2str(i) ',''XData'',[DFrameNo DFrameNo]/handles.datasamp);'])
    end
    %Store the updated variables back in the handles structure
    set(handles.DFrameNo,'String',num2str(DFrameNo));
end
% Update handles structure
guidata(hObject, handles);


function DFrameNo_Callback(hObject, eventdata, handles)
% hObject    handle to DFrameNo (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of DFrameNo as text
%        str2double(get(hObject,'String')) returns contents of DFrameNo as a double

%Create easy to use variables
DMaxFrame=handles.MyVars.DMaxFrame;
DFrameNo=str2double(get(handles.DFrameNo,'String'));
%Check for valid frame and update vertical data time bar
if (DFrameNo<1)
    DFrameNo=1;
else if (DFrameNo>DMaxFrame)
        DFrameNo=DMaxFrame;
    end
end
%Reposition vertical time bar in all data plots
for i=4:6
    eval(['set(handles.Bar' num2str(i) ',''XData'',[DFrameNo DFrameNo]/handles.datasamp);'])
end
%Store the updated variables back in the handles structure
set(handles.DFrameNo,'String',num2str(DFrameNo));
% Update handles structure
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function DFrameNo_CreateFcn(hObject, eventdata, handles)
% hObject    handle to DFrameNo (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function DJumpNum_Callback(hObject, eventdata, handles)
% hObject    handle to DJumpNum (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of DJumpNum as text
%        str2double(get(hObject,'String')) returns contents of DJumpNum as a double


% --- Executes during object creation, after setting all properties.
function DJumpNum_CreateFcn(hObject, eventdata, handles)
% hObject    handle to DJumpNum (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in DJumpBut.
function DJumpBut_Callback(hObject, eventdata, handles)
% hObject    handle to DJumpBut (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%Create easy to use variables
DMaxFrame=handles.MyVars.DMaxFrame;
DFrameNo=str2double(get(handles.DFrameNo,'String'));
DJumpNum=str2double(get(handles.DJumpNum,'String'));
%Check for valid frame and update vertical data time bar
DFrameNo=DFrameNo+DJumpNum;
if (DFrameNo<1)
    DFrameNo=1;
end
if (DFrameNo>DMaxFrame)
    DFrameNo=DMaxFrame;
end
for i=4:6
    %Reposition vertical time bar in all data plots
    eval(['set(handles.Bar' num2str(i) ',''XData'',[DFrameNo DFrameNo]/handles.datasamp);'])
end
%Store the updated variables back in the handles structure
set(handles.DFrameNo,'String',num2str(DFrameNo));
set(handles.DJumpNum,'String','0');
% Update handles structure
guidata(hObject, handles);




% --------------------------------------------------------------------
function Untitled_1_Callback(hObject, eventdata, handles)
% hObject    handle to Untitled_1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)




% --- Executes on button press in DoneBut.
function DoneBut_Callback(hObject, eventdata, handles)
% hObject    handle to DoneBut (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%NOTE: This is needed to resume the terminate the opening function execution
% (vidsync_vidgui_OpeningFcn) and run the closing function (vidsync_vidgui_OutputFcn)
% so that the main function (VIDSYNC_VIDGUI) can conclude
uiresume;
